/*
 * Copyright (c) 2007 The DragonFly Project.  All rights reserved.
 * 
 * This code is derived from software contributed to The DragonFly Project
 * by Matthew Dillon <dillon@backplane.com>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of The DragonFly Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific, prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $DragonFly: src/sys/sys/sysid.h,v 1.1 2007/04/29 01:29:31 dillon Exp $
 */

#ifndef _SYS_SYSID_H_
#define _SYS_SYSID_H_

/*
 * SYSIDs are 64 bit entities and come in two varieties.  Physical SYSIDs
 * are used to efficiently route messages across the mesh, while Logical
 * SYSIDs are persistently assigned identifiers representing specific devices
 * or specific media or named filesystems.  That is, the logical SYSID
 * assigned to a filesystem or ANVIL partition can be stored in that
 * filesystem's superblock and allows the filesystem to migrate or
 * be multi-homed (have multiple physical SYSIDs representing the same
 * logical entity).
 * 
 * Physical SYSIDs can be ever-changing, and any given logical SYSID could
 * in fact have multiple physical SYSIDs associated with it.  The mesh is
 * self-healing and the combination of the logical and physical sysid
 * basically validates the message at the target and determines whether
 * the physical SYSID must be recalculated (looked up again) or not.
 */

typedef u_int64_t	sysid_t;

#endif
